/******************************************************************************
 SyGChangeLogText.cc

	Copyright © 1998 by Glenn Bach. All rights reserved.

 ******************************************************************************/

#include "SyGHelpText.h"

const JCharacter* kChangeLogName  = "SyGChanges";
const JCharacter* kChangeLogTitle = "Changes from previous versions";

const JCharacter* kChangeLogText =

"Please send all bug reports and suggestions for features to Glenn Bach."
"<p>"
"<a href=mailto:glenn@its.caltech.edu>"
"<address>glenn@its.caltech.edu</address>"
"</a>"
"<p>"
"----------"
"<p>"
"<b>1.0.1</b>"
"<p>"

"Fixed a bug that caused a crash in certain circumstances when double-clicking."
"<p>"

"Fixed an updating problem."
"<p>"

"Fixed an updating problem relating to directories that no longer exist."

"<p>"
"<b>1.0</b>"
"<p>"
"initial public release"
;
