/******************************************************************************
 SyGSelectNameTask.h

	Interface for the SyGSelectNameTask class

	Copyright © 1996 by John Lindal. All rights reserved.

 ******************************************************************************/

#ifndef _H_SyGSelectNameTask
#define _H_SyGSelectNameTask

#include <JXIdleTask.h>
#include <JString.h>

class SyGFileTreeTable;

class SyGSelectNameTask : public JXIdleTask
{
public:

	SyGSelectNameTask(SyGFileTreeTable* table, const JCharacter* name);

	virtual ~SyGSelectNameTask();

	virtual void	Perform(const Time delta, Time* maxSleepTime);

private:

	SyGFileTreeTable*	itsTable;		// we don't own this
	JString				itsName;

private:

	// not allowed

	SyGSelectNameTask(const SyGSelectNameTask& source);
	const SyGSelectNameTask& operator=(const SyGSelectNameTask& source);
};

#endif
