/******************************************************************************
 SyGFileTreeList.h

	Interface for the SyGFileTreeList class

 ******************************************************************************/

#ifndef _H_SyGFileTreeList
#define _H_SyGFileTreeList

#include "JBroadcaster.h"

#include <JPtrArray.h>
#include <JRunArray.h>
#include "JNamedTreeList.h"

class JString;
class SyGFileTree;

class SyGFileTreeList : public JNamedTreeList
{
public:

	SyGFileTreeList(SyGFileTree* tree);
	
	virtual ~SyGFileTreeList();
	
	void UpdateList();
	void ForceListUpdate();

protected:

	virtual void Receive(JBroadcaster* sender, const Message& message);

private:

	SyGFileTree* itsFileTree;
	
private:

	// not allowed

	SyGFileTreeList(const SyGFileTreeList& source);
	const SyGFileTreeList& operator=(const SyGFileTreeList& source);

};

#endif
