/******************************************************************************
 SyGTOCHelpText.cc

	Copyright © 1999 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#include "SyGHelpText.h"

const JCharacter* kTOCHelpName  = "SyGTOCHelp";
const JCharacter* kTOCHelpTitle = "Table of Contents";

const JCharacter* kTOCHelpText =

"<html>"
"<h3>Table of Contents</h3>"

"<ul>"
"<li> <a href=jxhelp:SyGGettingStartedHelp>Getting started</a>"
"<p>"
"<li> <a href=jxhelp:SyGFolderHelp>Folders</a>"
"<p>"
"<ul>"
"<li> <a href=jxhelp:SyGFolderHelp#Opening>Opening folders</a>"
"<li> <a href=jxhelp:SyGFolderHelp#Shortcuts>Shortcuts</a>"
"<li> <a href=jxhelp:SyGFolderHelp#Creating>Creating folders</a>"
"<li> <a href=jxhelp:SyGFolderHelp#Views>View options</a>"
"<li> <a href=jxhelp:SyGFolderHelp#Filters>Filters</a>"
"</ul>"
"<p>"
"<li> <a href=jxhelp:SyGFileHelp>Files</a>"
"<p>"
"<ul>"
"<li> <a href=jxhelp:SyGFileHelp#MoveCopy>Moving and copying files</a>"
"<li> <a href=jxhelp:SyGFileHelp#Changing>Changing file names</a>"
"<li> <a href=jxhelp:SyGFileHelp#Duplicating>Duplicating files</a>"
"<li> <a href=jxhelp:SyGFileHelp#Deleting>Deleting files</a>"
"<li> <a href=jxhelp:SyGFileHelp#Symbolic>Symbolic links or aliases</a>"
"<li> <a href=jxhelp:SyGFileHelp#Locating>Locating files in a folder</a>"
"<li> <a href=jxhelp:SyGFileHelp#Permissions>File permissions</a>"
"<li> <a href=jxhelp:SyGFileHelp#Opening>Opening files</a>"
"<ul>"
"<p>"
"<li> <a href=jxhelp:SyGBindingEditorHelp>Bindings editor</a>"
"<p>"
"</ul>"
"<li> <a href=jxhelp:SyGMiscHelp#Run>Running programs</a>"
"</ul>"
"<p>"
"<li> <a href=jxhelp:SyGMiscHelp>Miscellaneous</a>"
"<p>"
"<ul>"
"<li> <a href=jxhelp:SyGMiscHelp#ViewMan>Viewing man pages</a>"
"<li> <a href=jxhelp:SyGMiscHelp#Term>Opening a terminal</a>"
"<li> <a href=jxhelp:SyGMiscHelp#Run>Running an arbitrary program</a>"
"<li> <a href=jxhelp:SyGKeyShortcutHelp>Useful keyboard and mouse shortcuts</a>"
"</ul>"
"<p>"
"<li> <a href=jxhelp:SyGChanges>Changes</a>"
"<p>"
"<li> <a href=jxhelp:SyGCredits>Credits</a>"
"<p>"
"</ul>"
"For more information about a particular feature, simply click on the underlined text."

"</html>"
;