/******************************************************************************
 SyGHelpText.h

	Copyright © 1998 by Glenn Bach. All rights reserved.

 ******************************************************************************/

#ifndef _H_SyGHelpText
#define _H_SyGHelpText

#include <jTypes.h>

extern JSize kHelpSectionCount;

extern const JCharacter* kHelpSectionName[];
extern const JCharacter* kHelpSectionTitle[];
extern const JCharacter* kHelpSectionText[];

extern const JCharacter* kTOCHelpName;
extern const JCharacter* kTOCHelpTitle;
extern const JCharacter* kTOCHelpText;

extern const JCharacter* kFolderHelpName;
extern const JCharacter* kFolderHelpTitle;
extern const JCharacter* kFolderHelpText;

extern const JCharacter* kFileHelpName;
extern const JCharacter* kFileHelpTitle;
extern const JCharacter* kFileHelpText;

extern const JCharacter* kBindingEditorHelpName;
extern const JCharacter* kBindingEditorHelpTitle;
extern const JCharacter* kBindingEditorHelpText;

extern const JCharacter* kMiscHelpName;
extern const JCharacter* kMiscHelpTitle;
extern const JCharacter* kMiscHelpText;
extern const JCharacter* kMiscManPageHelpName;
extern const JCharacter* kMiscRunHelpName;

extern const JCharacter* kGettingStartedHelpName;
extern const JCharacter* kGettingStartedHelpTitle;
extern const JCharacter* kGettingStartedHelpText;

extern const JCharacter* kKeyShortcutName;
extern const JCharacter* kKeyShortcutTitle;
extern const JCharacter* kKeyShortcutText;

extern const JCharacter* kChangeLogName;
extern const JCharacter* kChangeLogTitle;
extern const JCharacter* kChangeLogText;

extern const JCharacter* kCreditsName;
extern const JCharacter* kCreditsTitle;
extern const JCharacter* kCreditsText;

#endif