/******************************************************************************
 SyGChooseSavePrefObject.h

	Copyright © 1999 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#ifndef _H_SyGChooseSavePrefObject
#define _H_SyGChooseSavePrefObject

#include <JPrefObject.h>	

class SyGChooseSavePrefObject : public JPrefObject
{
public:

	SyGChooseSavePrefObject(JPrefsManager* prefsMgr, const JPrefID& id);

	virtual ~SyGChooseSavePrefObject();

protected:

	virtual void	ReadPrefs(istream& input);
	virtual void	WritePrefs(ostream& output) const;


private:

	// not allowed

	SyGChooseSavePrefObject(const SyGChooseSavePrefObject& source);
	const SyGChooseSavePrefObject& operator=(const SyGChooseSavePrefObject& source);
};

#endif
