/******************************************************************************
 GSimpleFileTree.h

	Interface for the GSimpleFileTree class

 ******************************************************************************/

#ifndef _H_GSimpleFileTree
#define _H_GSimpleFileTree

#include "JTree.h"
#include "JString.h"

class JTreeList;
class GSimpleFileTreeNode;

class GSimpleFileTree : public JTree
{
public:

	GSimpleFileTree(GSimpleFileTreeNode* root);
	
	virtual ~GSimpleFileTree();
	
	void SetTreeList(JTreeList* list);

	JTreeList* GetTreeList();
	
private:
	
	JTreeList* 	itsList;
	
private:

	// not allowed

	GSimpleFileTree(const GSimpleFileTree& source);
	const GSimpleFileTree& operator=(const GSimpleFileTree& source);

};

#endif
