/******************************************************************************
 SyGMainHelpText.cc

	Copyright © 1998 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#include "SyGHelpText.h"

const JCharacter* kFolderHelpName  	= "SyGFolderHelp";
const JCharacter* kFolderHelpTitle 	= "Folders";

const JCharacter* kFolderHelpText =

"<html>"
"<h3>Opening folders</h3>"
"<a name=Opening>"

"A folder, or directory, contains a collection of files and other folders. "
"There are several ways of opening a folder in SystemG. "
"<p> "

"Once SystemG is running, you can open a folder by double-clicking on its "
"icon. This will open the folder in a separate window. If you hold down the "
"Meta key while clicking, the current contents of the window will be "
"replaced by the contents of the clicked folder. If the folder is selected, "
"you can also open it with the keyboard in several ways. "
"<p>"

"Pressing Right-Arrow will open the selected folder while still showing "
"the current folder, and pressing Meta-Right-Arrow will open the selected "
"folder recursively (ie. all of its sub-folders will also be opened)"
". Pressing Meta-Down-Arrow will replace the current "
"window's contents with the contents of the selected folder. Pressing "
"Meta-Left-Arrow will close the selected folder if it is open, and Pressing "
"Meta-Up-Arrow will replace the contents of the window with the contents of "
"the current folder's parent folder. "
"<p>"

"Running SystemG from the command-line opens all of the folders that were "
"opened when you last quit the program (if they still exist). Once it is "
"running, you can also call SystemG from the command-line with arguments to "
"open other folders. This will not run a separate instance of SystemG, but "
"will pass the arguments to the currently running instance and it will open "
"the requested folders. If no arguments are specified on the command-line, "
"SystemG will open the folder that the command-line is currently in. "
"<p> "

"You can also open a folder with the <i>Open</i> command in the <i>File</i> "
"menu. If a folder is selected, it SystemG will open that folder. If nothing "
"is selected, an open dialog box will promtp for a folder to open. "
"<p> "

"Finally, you can quickly open a folder via the shortcut menu if it has "
"previously been designated as a shortcut."

"<h3>Shortcuts</h3>"
"<a name=Shortcuts>"

"Shortcuts provide a way to quickly access a folder. Once you have opened a "
"folder that you will be opening frequently, you can add it to the shortcut "
"list by selecting the <i>Add shortcut</i> from the <i>Shortcuts</i> menu. "
"Later when you wish to open that folder, simply select it from the list in "
"the <i>Shortcuts</i> menu. "

"<h3>Creating folders</h3>"
"<a name=Creating>"

"You can create a new folder by selecting the <i>New folder</i> command "
"from the <i>File</i> menu. If nothing is selected, a folder will be "
"created in the current folder. If a file is created, a folder will be "
"created in the folder containing that file. If a folder is created, a "
"folder will be created within that folder. "

"<h3>View options</h3>"
"<a name=Views>"

"There are several file properties that can be shown in the window by "
"selecting the appropriate menu item in the <i>Views</i> menu. The "
"properties available are <i>user</i>, <i>group</i>, <i>size</i>, "
"<i>permissions</i>, and <i>date</i>. Selecting <i>Show hidden</i> from the "
"<i>View</i> menu will show all of the hidden files with their properties as "
"well. "


"<h3>Filters</h3>"
"<a name=Filters>"

"There are times when there are too many files visible, making it hard to "
"find a particular file or group of files. The filter field is helpful in "
"this. The filter is made visible by selecting <i>Show filter</i> from the "
"<i>View</i> menu. The files are filtered based on the pattern in the filter "
"field. Patterns are constructed by replacing letters or groups of letters "
"in a file name with a '*'. For example, \"*.cc\" displays all files the end "
"in \".cc\". "

;