/******************************************************************************
 SyGKeyShortcutText.cc

	Copyright © 1999 by Glenn Bach. All rights reserved.

 ******************************************************************************/

#include "SyGHelpText.h"

const JCharacter* kKeyShortcutName  = "SyGKeyShortcutHelp";
const JCharacter* kKeyShortcutTitle = "Useful keyboard and mouse shortcuts";

const JCharacter* kKeyShortcutText =

"To really take advantage of SystemG, it is useful to be familiar with "
"the many keyboard and mouse shortcuts. These are described below."
"<p>"
"For many systems, the <i>Meta</i> key is labeled <i>Alt</i>"
"<p>"
"<blockquote>"
"<dl>"
"<dt> Return if one cell selected"
"<dd> Start editing the name of the selected cell"
"<p>"
"<dt> Return when editing"
"<dd> accept the name and stop editing"
"<p>"
"<dt> Escape"
"<dd> Cancel editing"
"<p>"
"<dt> Left-click on icon"
"<dd> Deselect all, then select the cell"
"<p>"
"<dt> Left-click on open space"
"<dd> Deselect all"
"<p>"
"<dt> Right-click on icon"
"<dd> Add current cell to the selection"
"<p>"
"<dt> Left-click and drag on open space"
"<dd> Select all cells contained in the selection rectangle"
"<p>"
"<dt> Double-click on file icon"
"<dd> Open file - same as <i>Open</i> on the <i>File</i> menu"
"<p>"
"<dt> Meta-double-click on file icon"
"<dd> Open file with alternate program"
"<p>"
"<dt> Ctrl-double-click on file icon"
"<dd> Open file with run dialog box"
"<p>"
"<dt> Double-click on folder icon"
"<dd> Open folder in new window - same as <i>Open</i> on the <i>File</i> menu"
"<p>"
"<dt> Meta-double-click on folder icon"
"<dd> Open folder in the same window - same as <i>Open</i> on the <i>File</i> menu"
"<p>"
"<dt> Right-arrow"
"<dd> Open selected folders"
"<p>"
"<dt> Meta-Right-arrow"
"<dd> Open selected folders and all of their sub-folders"
"<p>"
"<dt> Left-arrow"
"<dd> Close selected folders"
"<p>"
"<dt> Meta-Left-arrow"
"<dd> Close selected folders and all of their sub-folders"
"<p>"
"<dt> Meta-Down-arrow if one sub-folder selected"
"<dd> Move to the selected folder"
"<p>"
"<dt> Meta-Shift-Down-arrow if one sub-folder selected"
"<dd> Open new window for the selected folder"
"<p>"
"<dt> Meta-Up-arrow"
"<dd> Change directory to the parent folder"
"<p>"
"<dt> Meta-Shift-Up-arrow"
"<dd> Open new window for the parent folder"
"</dl>"
"</blockquote>"
;
