/******************************************************************************
 SyGMDIServer.h

	Interface for the SyGMDIServer class

	Copyright © 1997 by Glenn W. Bach.  All rights reserved.
	
	Base code generated by Codemill v0.1.0

 *****************************************************************************/

#ifndef _H_SyGMDIServer
#define _H_SyGMDIServer

#include <JXMDIServer.h>

class SyGApplication;

class SyGMDIServer : public JXMDIServer
{
public:

	SyGMDIServer(SyGApplication* app);
	virtual ~SyGMDIServer();

protected:

	virtual void	HandleMDIRequest(const JCharacter* dir,
									 const JPtrArray<JString>& argList);

//	virtual void	Receive(JBroadcaster* sender, const Message& message);

private:

	SyGApplication*	itsApp;

private:

	// not allowed

	SyGMDIServer(const SyGMDIServer& source);
	const SyGMDIServer& operator=(const SyGMDIServer& source);

};

#endif
