/******************************************************************************
 SyGEditPrefsDialog.h

	Interface for the SyGEditPrefsDialog class

	Copyright © 1997 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#ifndef _H_SyGEditPrefsDialog
#define _H_SyGEditPrefsDialog

#include <JXDialogDirector.h>
#include <JPtrArray.h>

class JString;
class JXInputField;
class JXRadioGroup;
class JXTextCheckbox;

class SyGEditPrefsDialog : public JXDialogDirector
{
public:

	SyGEditPrefsDialog(JXDirector* supervisor,
						  const JCharacter* terminalCmd,
						  const JCharacter* manViewCmd,
						  const JBoolean defMove,
						  const JBoolean bsTrash,
						  const JBoolean warn);

	virtual ~SyGEditPrefsDialog();

	void	GetPrefs(JString* terminalCmd, JString* manViewCmd, 
					 JBoolean* defMove, JBoolean* bsTrash,
					 JBoolean* warn) const;

private:

// begin JXLayout

    JXInputField*   itsManInput;
    JXInputField*   itsTerminalInput;
    JXRadioGroup*   itsDefAction;
    JXRadioGroup*   itsBSAction;
    JXTextCheckbox* itsWarnBeforeTrashCB;

// end JXLayout

private:

	void	BuildWindow(const JCharacter* terminalCmd, 
						const JCharacter* manViewCmd,
  					  	const JBoolean defMove,
					  	const JBoolean bsTrash,
					  	const JBoolean warn);

	// not allowed

	SyGEditPrefsDialog(const SyGEditPrefsDialog& source);
	const SyGEditPrefsDialog& operator=(const SyGEditPrefsDialog& source);
};

#endif
