/******************************************************************************
 SyGFileHelpText.cc

	Copyright © 1999 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#include "SyGHelpText.h"

const JCharacter* kFileHelpName  	= "SyGFileHelp";
const JCharacter* kFileHelpTitle 	= "Files";

const JCharacter* kFileHelpText =

"<html>"
"<h3>Moving and copying files</h3>"
"<a name=MoveCopy>"

"Files are moved or copied by dragging the file from one location to "
"another. That location may be in the same window or not. If the file is "
"placed over a sub-folder in a window while being dragged, that folder will "
"darken to indicate that it is the target. By default, files will be moved "
"when they are dragged. To copy the file, hold down the <i>Meta</i> key "
"while dragging. If the <i>Control</i> key is instead held down, you will be "
"asked whether you want to move or copy the file."

"<h3> Changing file names </h3>"
"<a name=Changing>"
 
"To change the name of a file, you can either left-click on the file name, "
"or select the file and press the return key."

"<h3> Duplicating files </h3>"
"<a name=Duplicating>"
 
"To duplicate a file or group of files, select them and choose "
"<i>Duplicate</i> from the <i>File</i> menu."

"<h3> Deleting files </h3>"
"<a name=Deleting>"
 
"When you want to remove a file you have two choices. You can either move "
"the file to the trash, or simple delete it. Moving a file to the trash "
"allows you to change your mind later, since you can retrieve the file from "
"the trash by selecting <i>Open trash</i> from the <i>File</i> menu, and "
"dragging the file out of the trash. The trash can be emptied by selecting "
"<i>Empty trash</i> from the <i>File</i> menu. When you select <i>Delete</i> "
"from the <i>File</i> menu, the file will be permanently removed from the "
"hard disk."

"<h3> Symbolic links or aliases </h3>"
"<a name=Symbolic>"
 
"A symbolic link appears to be a file, but is actully just a pointer to a "
"file in another location. The symbolic link can have any name. The "
"advantage of a symbolic link is that you can have access to a file or "
"folder from many locations without actually duplicating it several times. "
"This keeps your disk usage down, and forces the file or folder to remain "
"the same at each location."

"<p>"
 
"You create a symbolic link by selecting the file and choosing <i>Create "
"link</i> from the <i>View</i> menu. The resultant link can be moved to "
"another location, and its name can be changed. If you have a symbolic link, "
"you can determine what it points to be seleting <i>Find link</i> from the "
"<i>View</i> menu."

"<h3> Locating files in a folder </h3>"
"<a name=Locating>"
 
"To locate a file within a window, simply start typing its name. As the "
"letters you type start matching a word, that word will be selected. "
"To clear the selection, or to search for a new word, hit the space key."

"<h3> File permissions </h3>"
"<a name=Permissions>"
 
"The permissions on a file or folder determine who can read, write, or "
"execute a file. The permissions are listed from left to right for the owner "
"of the file, the owner's group, and for everyone else. To change the "
"permissions of a file or folder, toggle the permission you want to change. "
"For example, If a file is not currently writable by anyone, and you want to "
"be able to write to it, click the write flag which will be the second digit "
"from the left. Clicking it again will change it back. To change the "
"permissions on every file in a directory, <i>Meta</i> click the appropriate "
"permission on the folder. "

"<h3> Opening files </h3>"
"<a name=Opening>"
 
"To open a file, either double-click on its icon, or select it and choose "
"<i>Open</i> from the <i>File</i> menu. The program that opens it will be "
"determined by the <a href=jxhelp:SyGBindingEditorHelp>Bindings editor</a>. "

"The bindings editor contains up to two programs per extension, the standard "
"and the alternate. If neither of these is appropriate at the time, you may "
"open the file by holding the <i>Control</i> button down while "
"double-clicking on the file. You will then be presented with the "
"<a href=jxhelp:SyGMiscHelp#Run>Run command dialog</a> which will allow you "
"to specify an arbitrary program to with which to open the file. "

 ;