/******************************************************************************
 SyGRemoteErrUpdateTask.h

	Interface for the SyGRemoteErrUpdateTask class

	Copyright © 1999 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#ifndef _H_SyGRemoteErrUpdateTask
#define _H_SyGRemoteErrUpdateTask

#include <JXUrgentTask.h>

class SyGRemoteErrUpdateTask : public JXUrgentTask
{
public:

	SyGRemoteErrUpdateTask();

	virtual ~SyGRemoteErrUpdateTask();

	virtual void	Perform();

private:

	// not allowed

	SyGRemoteErrUpdateTask(const SyGRemoteErrUpdateTask& source);
	const SyGRemoteErrUpdateTask& operator=(const SyGRemoteErrUpdateTask& source);
};

#endif
