/******************************************************************************
 SyGGettingStartedHelpText.cc

	Copyright © 1998 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#include "SyGHelpText.h"

const JCharacter* kGettingStartedHelpName  = "SyGGettingStartedHelp";
const JCharacter* kGettingStartedHelpTitle = "Getting started with SystemG";

const JCharacter* kGettingStartedHelpText =

"<html>"
"<h3>GettingStarted with the SystemG</h3>"
 
"SystemG is a simple file manager that provides basic access to the file "
"system. The goal of the design was to allow an intuitive way for the user "
"to access and manipulate their files and folders. Files and folders are "
"opened by double-clicking on them, names are changed by clicking on them "
"and then typing in the new name, permissions are changed by clicking on the "
"appropriate permission, etc."

"<p>"
 
"Running SystemG for the first time will open your home directory. You can "
"then navigate through your files by using the <i>Open</i> command in the "
"<i>File</i> menu, or double-clicking on a directory. If you open a "
"directory that you expect to access frequently, you can add it to the "
"shortcut list, by choosing <i>Add shortcut</i> from the <i>Shortcuts</i> "
"menu."

"<p>"
 
"As an example of the ease of use of SystemG, let's suppose that you needed "
"to edit the name of a file in a particular directory. You would first use "
"the <i>Open</i> command (or <i>Meta-O</i>), and select the appropriate "
"directory, then you would start typing the letters in the old name, which "
"would locate the file in the directory and select it. If you then hit the "
"<i>Return</i> key, the file name would become editable. You then type in "
"the new name and press the <i>Return</i> key again."

;