/******************************************************************************
 SyGHelpText.cc

	Copyright © 1998 by Glenn Bach. All rights reserved.

 ******************************************************************************/

#include "SyGHelpText.h"

const JCharacter* kHelpSectionName[] =
	{ kFolderHelpName, kFileHelpName, kBindingEditorHelpName, 
		kGettingStartedHelpName, kChangeLogName, kCreditsName,
		kMiscHelpName, kKeyShortcutName, kTOCHelpName};

const JCharacter* kHelpSectionTitle[] =
	{ kFolderHelpTitle, kFileHelpTitle, kBindingEditorHelpTitle, 
		kGettingStartedHelpTitle, kChangeLogTitle, kCreditsTitle,
		kMiscHelpTitle, kKeyShortcutTitle, kTOCHelpTitle };

const JCharacter* kHelpSectionText[] =
	{ kFolderHelpText, kFileHelpText, kBindingEditorHelpText, 
		kGettingStartedHelpText, kChangeLogText, kCreditsText,
		kMiscHelpText, kKeyShortcutText, kTOCHelpText };

JSize kHelpSectionCount = sizeof(kHelpSectionName) / sizeof(JCharacter*);
