#ifndef _H_FMDNDCursor
#define _H_FMDNDCursor

static const JCharacter* kFMDNDCursorName = "FMDNDCursor";

static char file_dnd_cursor_bits[] = {
   0x00, 0x00, 0xfe, 0x00, 0x02, 0x01, 0x02, 0x02, 0x02, 0x04, 0x02, 0x0f,
   0x02, 0x08, 0x42, 0x08, 0xc2, 0x08, 0xc2, 0x09, 0xc2, 0x0b, 0xc2, 0x07,
   0xc2, 0x0f, 0xde, 0x1f, 0xc0, 0x3f, 0xc0, 0x07, 0xc0, 0x06, 0x40, 0x0c,
   0x00, 0x0c, 0x00, 0x18, 0x00, 0x18, 0x00, 0x00};

static char file_dnd_cursor_mask_bits[] = {
   0xff, 0x00, 0xff, 0x01, 0xff, 0x03, 0xff, 0x07, 0xff, 0x0f, 0xff, 0x1f,
   0xff, 0x1f, 0xff, 0x1f, 0xff, 0x1f, 0xff, 0x1f, 0xff, 0x1f, 0xff, 0x1f,
   0xff, 0x1f, 0xff, 0x7f, 0xff, 0x7f, 0xe0, 0x7f, 0xe0, 0x0f, 0xe0, 0x1e,
   0x60, 0x1e, 0x00, 0x3c, 0x00, 0x3c, 0x00, 0x18};

static const JXCursor kFMDNDCursor =
{
	15,22, 6,7, file_dnd_cursor_bits, file_dnd_cursor_mask_bits
};	

#endif