/******************************************************************************
 SyGFileTreeUpdateTask.h

	Interface for the SyGFileTreeUpdateTask class

	Copyright © 1996 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#ifndef _H_SyGFileTreeUpdateTask
#define _H_SyGFileTreeUpdateTask

#include <JXIdleTask.h>

class SyGFileTreeTable;

class SyGFileTreeUpdateTask : public JXIdleTask
{
public:

	SyGFileTreeUpdateTask(SyGFileTreeTable* theTreeTable);

	virtual ~SyGFileTreeUpdateTask();

	virtual void	Perform(const Time delta, Time* maxSleepTime);

private:

	SyGFileTreeTable*	itsTreeTable;		// we don't own this
	Time			itsElapsedTime;

private:

	// not allowed

	SyGFileTreeUpdateTask(const SyGFileTreeUpdateTask& source);
	const SyGFileTreeUpdateTask& operator=(const SyGFileTreeUpdateTask& source);
};

#endif
