/******************************************************************************
 SyGMiscHelpText.cc

	Copyright © 1999 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#include "SyGHelpText.h"

const JCharacter* kMiscHelpName  		= "SyGMiscHelp";
const JCharacter* kMiscHelpTitle 		= "Miscellaneous help";

const JCharacter* kMiscManPageHelpName 	= "SyGMiscHelp#ViewMan";
const JCharacter* kMiscRunHelpName  	= "SyGMiscHelp#Run";

const JCharacter* kMiscHelpText =

"<html>"

"<h3> Viewing UNIX Man Pages </h3>"
"<a name=ViewMan>"

"The Man Pages window lets you specify what function to look up and which "
"section to look in.  If you leave the section blank, the system will search "
"all sections."
"<p>"
"To search the man pages for all occurrences of the function, check the Apropos "
"checkbox."
"<p>"
"You can get the Man Pages window via the <i>View man page</i> menu item on the "
"<i>File</i> menu."

"<p>"

"<h3> Opening a terminal </h3>"
"<a name=Term>"

"There are times when the easiest way to do something is with a command-line "
"terminal. To open a terminal, select <i>Open terminal</i> from the "
"<i>File</i> menu. This will open a terminal in the current window\'s "
"directory. By default, the terminal command called is <i>xterm</i>, but you "
"can change that in the Miscellaneous preferences. "

"<p>"

"<h3> Running an aritrary program </h3>"
"<a name=Run>"

"If you select <i>Run command...</i> from the <i>File</i> menu, you will be "
"presented with a dialog box containing an input field. You can type any "
"command that is available to you on the system. Commands that you enter "
"will be stored on the history menu to the right of the input field. If you "
"don\'t remember the name of a command, you can click on the <i>Browse</i> "
"button to look through the programs available. "

"<p>"

"You can also access this dialog box by <i>control-double-clicking</i> on a "
"file. When the dialog is then presented to you, you specify the program to "
"run with <i>$f</i> representing the file name. "

"<p>"

"If you just want to run a program that is visible in the current window, "
"you can simply <i>double-click</i> on the program and it will be run. "

"</html>"
;
