/******************************************************************************
 PatterBinding.h

	Interface for the PatterBinding class

	Copyright © 1997 by Glenn Bach. 

 ******************************************************************************/

#ifndef _H_PatterBinding
#define _H_PatterBinding

#include <JCharacter.h>
#include <JString.h>

class PatterBinding
{
public:

	PatterBinding(const JCharacter* pattern, const JCharacter* action, 
					const JCharacter* alternate);
	virtual ~PatterBinding();
	
	const JString& 	GetPattern() const;
	void			SetPattern(const JCharacter* pattern);
	
	const JString& 	GetAction() const;
	void			SetAction(const JCharacter* action);
	
	const JString& 	GetAlternate() const;
	void			SetAlternate(const JCharacter* alternate);

private:

	JString		itsPattern;
	JString		itsAction;
	JString		itsAlternate;
	
	// not allowed

	PatterBinding(const PatterBinding& source);
	const PatterBinding& operator=(const PatterBinding& source);


}

#endif