/******************************************************************************
 SyGTreeDir.h

	Interface for the SyGTreeDir class

	Copyright © 1997 by Glenn Bach. All rights reserved.

 ******************************************************************************/

#ifndef _H_SyGTreeDir
#define _H_SyGTreeDir

#include <JXWindowDirector.h>

class JString;
class SyGSet;
class ostrstream;
class JXImage;

class SyGTreeDir : public JXWindowDirector
{
public:

	SyGTreeDir(JXDirector* supervisor, const JString& startPath, 
					JBoolean isMain, JCoordinate width = 400, 
					JCoordinate height = 300, JCoordinate locX = 100, 
					JCoordinate locY = 100, JCoordinate partWidth = 150);

	virtual ~SyGTreeDir();
	
	const JString& GetCurrentPath();
	virtual JBoolean Close();
	void SelectName(const JString& selectName);

protected:

	virtual void	Receive(JBroadcaster* sender, const Message& message);

private:

// begin JXLayout


// end JXLayout

	SyGSet* 	itsFMSet;
	JXImage*	itsWindowIcon;

private:

	void	BuildWindow(const JString& startPath, JBoolean isMain,
						JCoordinate width, 
						JCoordinate height, JCoordinate locX, 
						JCoordinate locY, JCoordinate partWidth);
	JBoolean itsIsMain;

	// not allowed

	SyGTreeDir(const SyGTreeDir& source);
	const SyGTreeDir& operator=(const SyGTreeDir& source);
};

#endif
